

#include "geometry.h"



//     ,     


//-----------------------------------------------------------------------
// class Point

void Point::Draw(FILE *f, const char *text) const
{
    if (text) fprintf(f, "#%s\n", text);
    if (!def) { 
        fprintf(f, "# not defined\n");
        return;
    }
    fprintf(f, " %10.6f %10.6f\n", x, y+0.1);
    fprintf(f, " %10.6f %10.6f\n", x-0.1, y);
    fprintf(f, " %10.6f %10.6f\n", x, y-0.1);
    fprintf(f, " %10.6f %10.6f\n", x+0.1, y);
    fprintf(f, " %10.6f %10.6f\n     \n", x, y+0.1);
}


std::ostream & operator<<(std::ostream & os, const Point &pt)
{
    os << " (" << pt.x << ", " << pt.y << ")";
    return os;
}



//-----------------------------------------------------------------------
// class Segment

void Segment::Draw(FILE *f, const char *text) const
{
    if (text) fprintf(f, "#%s\n", text);
    if (!def) { 
        fprintf(f, "# not defined\n");
        return;
    }
    fprintf(f, " %10.6f %10.6f\n", p1.x, p1.y);
    fprintf(f, " %10.6f %10.6f\n     \n", p2.x, p2.y);
}

//-----------------------------------------------------------------------
// class Circle

void Circle::Draw(FILE *f, const char *text) const
{
    if (text) fprintf(f, "#%s\n", text);
    if (!def) { 
        fprintf(f, "# not defined\n");
        return;
    }
    int n = 360;
    double h = 2*M_PI / n;
    double x1, y1, x2, y2;
    x1 = center.x + radius;
    y1 = center.y;
    for (int i=1; i<=n; i++) {
       x2 = center.x + radius*cos(i*h);
       y2 = center.y + radius*sin(i*h);
       fprintf(f, " %10.6f %10.6f\n %10.6f %10.6f\n", x1, y1, x2, y2);
       x1 = x2;
       y1 = y2;
    }
    fprintf(f, "     \n"); 
    center.Draw(f);
}

Segment Circle::operator&(const Circle &c) const
{
    Segment dcenters(center, c.center);
    double dist = dcenters.Length();

    if (dist < POINT_EPS && fabs(radius - c.radius) < POINT_EPS) return Segment(); 
    if (radius + c.radius < dist) return Segment();
    if (fabs(radius - c.radius) > dist) return Segment();
    
    double px = (dist*dist + radius*radius - c.radius*c.radius)/(2*dist);
    double py = (radius*radius - px*px);
    if (py < 0) return Segment();
    py = sqrt(py);
    double ux, uy, vx, vy;
    ux = (c.center.x - center.x)/dist;
    uy = (c.center.y - center.y)/dist;
    Point pm(center.x + px*ux, center.y + px*uy);
    vx = -uy;
    vy = ux;
    double d = sqrt(vx*vx + vy*vy);
    if (d < POINT_EPS) return Segment(pm, pm);
    vx /= d;
    vy /= d;
    return Segment(Point(pm.x + py*vx, pm.y + py*vy), Point(pm.x - py*vx, pm.y - py*vy));
}

//-----------------------------------------------------------------------
// class Triangle

void Triangle::Draw(FILE *f, const char *text) const
{
    if (text) fprintf(f, "#%s\n", text);
    if (!def) { 
        fprintf(f, "# not defined\n");
        return;
    }
    Segment(p1,p2).Draw(f, nullptr);
    Segment(p2,p3).Draw(f, nullptr);
    Segment(p3,p1).Draw(f, nullptr);

}
                 
