
#pragma once

//      
//       
//        
//         
//         


#include<cmath>
#include<cstdio>                 //  FILE*  ..
#include<iostream>               //  operator<<   ..

#define  POINT_EPS  1e-10        // ,    

class Line;                      //  
class Segment;                   // ,  
class Circle;                    // 
class Triangle;

//----------------------------------------------------------------------
class Point {
private:
    double x,y;    // 
    bool def;      //   (def - definite)
    //     
    double Dist(const Point &a) const { return sqrt((x-a.x)*(x-a.x) + (y-a.y)*(y-a.y)); }
public:                                                            
    Point() : def(false) {}                                       // #1
    Point(double xx, double yy) : x(xx), y(yy), def(true) {}  // #2
    Point(const Point &a) : x(a.x), y(a.y), def(a.def) {}         // #3
    ~Point() = default;                                           // #4
    bool IsDef() const { return def; }                            // #5
    const Point & operator=(const Point &a) { x = a.x; y = a.y; def = a.def; return *this; }
    bool operator==(const Point &a) const { return Dist(a) < POINT_EPS; }
 
    friend std::ostream & operator<<(std::ostream & os, const Point &pt);     // #6
    void Draw(FILE *f, const char *text = nullptr) const;         // #7

    friend class Segment;                  //      
    friend class Line;
    friend class Circle;
    friend class Triangle;
};

/*    

#1      ,        
     ,    (  - ),      

#2          (  NaN !)
        def  true  false

#3    ;    ,     , 
          

#4     ,     (),      
       

#5     , ..    

#6      <<  (  ,   ),
           

#7    , ,  gnuplot,   -   :)

*/

//----------------------------------------------------------------------
class Segment {
private:
    Point p1, p2;
    bool def;
    double Length() const { return p1.Dist(p2); }                // #1
public:
    Segment() : def(false) {}
    Segment(const Point &a1, const Point &a2) : p1(a1), p2(a2), def(a1.def && a2.def) {}
    Segment(const Segment &s) : p1(s.p1), p2(s.p2), def(s.def) {}
    bool IsDef() const { return def; }
    const Segment & operator=(const Segment &s) { p1 = s.p1; p2 = s.p2; def = s.def; return *this; }
    bool operator==(const Segment &s) const { return fabs(Length() - s.Length()) < POINT_EPS; }      // #2
    const Segment & operator-() { Point t(p1); p1 = p2; p2 = t; return *this; }                  // #3
    bool IsTrivial() const { return def && Length() < POINT_EPS; }                               // #4
    void Draw(FILE *f, const char *text = nullptr) const;
    const Point & First()  const { return p1; }
    const Point & Second() const { return p2; }
                                                       // #5
    friend class Circle;                                         

};

/*    

#1       ,    private

#2    ,     (       Segment)

#3     s    = -s

#4     -      (  !)

#5      operator<<
*/
//----------------------------------------------------------------------
class Circle {
private:
    Point center;
    double radius;
    bool def;
public:
    Circle() : def(false) {}
    Circle(const Segment &a) : center(a.p1), radius(a.Length()), def(a.def) {};    // #1
    Circle(const Circle &c);                                                       // #2
    bool IsDef() const { return def; }
    void MoveTo(const Point &a) { center = a; def = a.def; }                     // #3
    Segment operator&(const Circle &c) const;                // #4

    void Draw(FILE *f, const char *text = nullptr) const;
                                                             // #5  
};

/*    

#1    ,  -  ,  -  

#2    :)

#3      ;   ,     

#4    ,   

#5      operator<<   
*/

//----------------------------------------------------------------------
class Triangle {
private:
    Point p1, p2, p3;
    bool def;
public:
    Triangle (const Point &q1, const Point &q2, const Point &q3) : p1(q1), p2(q2), p3(q3), def(p1.def && p2.def && p3.def) {}
    void Draw(FILE *f, const char *text = nullptr) const;
};


